@@ -2,22 +2,23 @@
 /* vim:set et sts=4: */
 /* IBus - The Input Bus
  * Copyright (C) 2008-2010 Peng Huang <shawn.p.huang@gmail.com>
- * Copyright (C) 2008-2010 Red Hat, Inc.
+ * Copyright (C) 2024 Takao Fujiwara <takao.fujiwara1@gmail.com>
+ * Copyright (C) 2008-2024 Red Hat, Inc.
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public
  * License as published by the Free Software Foundation; either
- * version 2 of the License, or (at your option) any later version.
+ * version 2.1 of the License, or (at your option) any later version.
  *
  * This library is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.	 See the GNU
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
  * Lesser General Public License for more details.
  *
  * You should have received a copy of the GNU Lesser General Public
- * License along with this library; if not, write to the
- * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
- * Boston, MA 02111-1307, USA.
+ * License along with this library; if not, write to the Free Software
+ * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301
+ * USA
  */
 
 #include "matchrule.h"
@@ -198,30 +199,31 @@ failed:
 static gchar *
 find_value (const gchar **p)
 {
-    GString *text;
-
-    text = g_string_new ("");
-
     SKIP_WHITE (*p);
 
-    if (**p != '\'')
-        goto failed;
-    (*p) ++;
-
-    while (**p != '\'') {
-        if (**p == '\0')
-            goto failed;
-        if (**p == '\\')
-            (*p) ++;
-        g_string_append_c (text, **p);
+    if (**p == '\'') {
+        GString *text = g_string_new ("");
         (*p) ++;
+        while (**p != '\'') {
+            if (**p == '\0') {
+                g_string_free (text, TRUE);
+                return NULL;
+            }
+            if (**p == '\\')
+                (*p) ++;
+            g_string_append_c (text, **p);
+            (*p) ++;
+        }
+        (*p) ++;
+        return g_string_free (text, FALSE);
+    } else if (strncmp (*p, "true", 4) == 0) {
+        *p += 4;
+        return g_strdup ("true");
+    } else if (strncmp (*p, "false", 5) == 0) {
+        *p += 5;
+        return g_strdup ("false");
     }
-    (*p) ++;
 
-    return g_string_free (text, FALSE);
-
-failed:
-    g_string_free (text, TRUE);
     return NULL;
 }
 
@@ -372,6 +374,11 @@ bus_match_rule_new (const gchar *text)
                 goto failed;
             bus_match_rule_set_arg (rule, i, p->value);
         }
+        else if (g_strcmp0 (p->key, "eavesdrop") == 0) {
+            if (g_strcmp0 (p->value, "true") != 0 &&
+                g_strcmp0 (p->value, "false") != 0)
+                goto failed;
+        }
         else
             goto failed;
     }
@@ -401,6 +408,13 @@ bus_match_rule_set_message_type (BusMatchRule   *rule,
     return TRUE;
 }
 
+const gchar *
+bus_match_rule_get_sender  (BusMatchRule    *rule)
+{
+    g_return_val_if_fail (rule != NULL, NULL);
+    return rule->sender;
+}
+
 gboolean
 bus_match_rule_set_sender  (BusMatchRule    *rule,
                             const gchar     *sender)
