@@ -2,8 +2,8 @@
 /* vim:set et sts=4: */
 /* ibus - The Input Bus
  * Copyright (C) 2008-2013 Peng Huang <shawn.p.huang@gmail.com>
- * Copyright (C) 2015-2023 Takao Fujiwara <takao.fujiwara1@gmail.com>
- * Copyright (C) 2008-2023 Red Hat, Inc.
+ * Copyright (C) 2015-2025 Takao Fujiwara <takao.fujiwara1@gmail.com>
+ * Copyright (C) 2008-2025 Red Hat, Inc.
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public
@@ -27,6 +27,7 @@
 
 #include <string.h>
 #include <gtk/gtk.h>
+#include <gdk/gdk.h>
 #include <gdk/gdkkeysyms.h>
 #include <ibus.h>
 #include "ibusimcontext.h"
@@ -111,7 +112,7 @@ static guint    _signal_delete_surrounding_id = 0;
 static guint    _signal_retrieve_surrounding_id = 0;
 
 #if GTK_CHECK_VERSION (3, 98, 4)
-static char _use_sync_mode = 2;
+static char _use_sync_mode = 1;
 #else
 static const gchar *_no_snooper_apps = NO_SNOOPER_APPS;
 static gboolean _use_key_snooper = ENABLE_SNOOPER;
@@ -360,6 +361,12 @@ ibus_im_context_commit_event (IBusIMContext *ibusimcontext,
         keyval == GDK_KEY_KP_Enter) {
         return FALSE;
     }
+    /* #2588 If IBus tries to commit a character, it should be forwarded to
+     * the application at once with IBUS_IGNORED_MASK before the actual
+     * commit because any characters can be control characters even if
+     * they are not ASCII characters, e.g. game cursor keys with a
+     * language keyboard layout likes VIM cursor mode  "hjkl" keys.
+     */
     ch = ibus_keyval_to_unicode (keyval);
     if (ch != 0 && !g_unichar_iscntrl (ch)) {
         IBusText *text = ibus_text_new_from_unichar (ch);
@@ -386,6 +393,7 @@ typedef struct {
     gboolean  retval;
 } ProcessKeyEventReplyData;
 
+
 static void
 _process_key_event_done (GObject      *object,
                          GAsyncResult *res,
@@ -435,6 +443,7 @@ _process_key_event_done (GObject      *object,
 #endif
 }
 
+
 static void
 _process_key_event_reply_done (GObject      *object,
                                GAsyncResult *res,
@@ -457,6 +466,7 @@ _process_key_event_reply_done (GObject      *object,
     g_source_remove (data->count_cb_id);
 }
 
+
 static gboolean
 _process_key_event_count_cb (gpointer user_data)
 {
@@ -472,6 +482,101 @@ _process_key_event_count_cb (gpointer user_data)
     return G_SOURCE_CONTINUE;
 }
 
+
+static gboolean
+_process_key_event_sync (IBusInputContext *context,
+                         guint             keyval,
+                         guint             keycode,
+                         guint             state)
+{
+    gboolean retval;
+
+    g_assert (IBUS_IS_INPUT_CONTEXT (context));
+    retval = ibus_input_context_process_key_event (context,
+                                                   keyval,
+                                                   keycode - 8,
+                                                   state);
+    ibus_input_context_post_process_key_event (context);
+    return retval;
+}
+
+
+static gboolean
+_process_key_event_async (IBusInputContext *context,
+                          guint             keyval,
+                          guint             keycode,
+                          guint             state,
+                          GdkEvent         *event,
+                          IBusIMContext    *ibusimcontext)
+{
+    ProcessKeyEventData *data = g_slice_new0 (ProcessKeyEventData);
+
+    g_assert (event);
+    if (!data) {
+        g_warning ("Cannot allocate async data");
+        return _process_key_event_sync (context, keyval, keycode, state);
+    }
+#if GTK_CHECK_VERSION (3, 98, 4)
+    data->event = gdk_event_ref (event);
+#else
+    data->event = gdk_event_copy (event);
+#endif
+    data->ibusimcontext = ibusimcontext;
+    ibus_input_context_process_key_event_async (context,
+            keyval,
+            keycode - 8,
+            state,
+            -1,
+            NULL,
+            _process_key_event_done,
+            data);
+
+    return TRUE;
+}
+
+
+static gboolean
+_process_key_event_hybrid_async (IBusInputContext *context,
+                                 guint             keyval,
+                                 guint             keycode,
+                                 guint             state)
+{
+    GSource *source = g_timeout_source_new (1);
+    ProcessKeyEventReplyData *data = NULL;
+    gboolean retval = FALSE;
+
+    if (source)
+        data = g_slice_new0 (ProcessKeyEventReplyData);
+    if (!data) {
+        g_warning ("Cannot wait for the reply of the process key event.");
+        retval = _process_key_event_sync (context, keyval, keycode, state);
+        if (source)
+            g_source_destroy (source);
+        return retval;
+    }
+    data->count = 1;
+    g_source_attach (source, NULL);
+    g_source_unref (source);
+    data->count_cb_id = g_source_get_id (source);
+    ibus_input_context_process_key_event_async (context,
+            keyval,
+            keycode - 8,
+            state,
+            -1,
+            NULL,
+            _process_key_event_reply_done,
+            data);
+    g_source_set_callback (source, _process_key_event_count_cb, data, NULL);
+    while (data->count)
+        g_main_context_iteration (NULL, TRUE);
+    /* #2498 Checking source->ref_count might cause Nautilus hang up
+     */
+    retval = data->retval;
+    g_slice_free (ProcessKeyEventReplyData, data);
+    return retval;
+}
+
+
 static gboolean
 _process_key_event (IBusInputContext *context,
 #if GTK_CHECK_VERSION (3, 98, 4)
@@ -505,70 +610,20 @@ _process_key_event (IBusInputContext *context,
 
     switch (_use_sync_mode) {
     case 1: {
-        retval = ibus_input_context_process_key_event (context,
-                                                       keyval,
-                                                       keycode - 8,
-                                                       state);
+        retval = _process_key_event_sync (context, keyval, keycode, state);
         break;
     }
     case 2: {
-        GSource *source = g_timeout_source_new (1);
-        ProcessKeyEventReplyData *data = NULL;
-
-        if (source)
-            data = g_slice_new0 (ProcessKeyEventReplyData);
-        if (!data) {
-            g_warning ("Cannot wait for the reply of the process key event.");
-            retval = ibus_input_context_process_key_event (context,
-                                                           keyval,
-                                                           keycode - 8,
-                                                           state);
-            if (source)
-                g_source_destroy (source);
-            break;
-        }
-        data->count = 1;
-        g_source_attach (source, NULL);
-        g_source_unref (source);
-        data->count_cb_id = g_source_get_id (source);
-        ibus_input_context_process_key_event_async (context,
-            keyval,
-            keycode - 8,
-            state,
-            -1,
-            NULL,
-            _process_key_event_reply_done,
-            data);
-        g_source_set_callback (source, _process_key_event_count_cb, data, NULL);
-        while (data->count)
-            g_main_context_iteration (NULL, TRUE);
-        if (source->ref_count > 0) {
-            /* g_source_get_id() could causes a SEGV */
-            g_info ("Broken GSource.ref_count and maybe a timing issue in %p.",
-                    source);
-        }
-        retval = data->retval;
-        g_slice_free (ProcessKeyEventReplyData, data);
+        retval = _process_key_event_hybrid_async (context,
+                                                  keyval, keycode, state);
         break;
     }
     default: {
-        ProcessKeyEventData *data = g_slice_new0 (ProcessKeyEventData);
-#if GTK_CHECK_VERSION (3, 98, 4)
-        data->event = gdk_event_ref (event);
-#else
-        data->event = gdk_event_copy ((GdkEvent *)event);
-#endif
-        data->ibusimcontext = ibusimcontext;
-        ibus_input_context_process_key_event_async (context,
-            keyval,
-            keycode - 8,
-            state,
-            -1,
-            NULL,
-            _process_key_event_done,
-            data);
-
-        retval = TRUE;
+        retval = _process_key_event_async (context,
+                                           keyval, keycode, state,
+                                           (GdkEvent *)event,
+                                           ibusimcontext);
+        break;
     }
     }
 
@@ -877,7 +932,55 @@ ibus_im_context_class_init (IBusIMContextClass *class)
     g_assert (_signal_retrieve_surrounding_id != 0);
 
 #if GTK_CHECK_VERSION (3, 98, 4)
-    _use_sync_mode = _get_char_env ("IBUS_ENABLE_SYNC_MODE", 2);
+    /* IBus GtkIMModule, QtIMModlue, ibus-x11, ibus-wayland are called as
+     * IBus clients.
+     * Each GTK application, each QT application, Xorg server, Wayland
+     * comppsitor are called as IBus event owners here.
+     *
+     * The IBus client processes the key events between the IBus event owner
+     * and the IBus daemon and the procedure step is to:
+     *
+     * receive the key event from the IBus event owner and forward the
+     * event to the IBus daemon with the "ProcessKeyEvent" D-Bus method at
+     * first,
+     *
+     * receive the return value from the IBus daemon with the "ProessKeyEvent"
+     * D-Bus method and forward the value to the IBus event owner secondly and
+     * the return value includes if the key event is processed normally or not.
+     *
+     * The procedure behavior can be changed by the "IBUS_ENABLE_SYNC_MODE"
+     * environment variable with the synchronous procedure or asynchronous
+     * one and value is:
+     *
+     * 1: Synchronous process key event:
+     *    Wait for the return of the IBus "ProcessKeyEvent" D-Bus method
+     *    synchronously and forward the return value to the IBus event owner
+     *    synchronously.
+     * 0: Asynchronous process key event:
+     *    Return to the IBus event owner as the key event is processed normally
+     *    at first as soon as the IBus client receives the event from the
+     *    IBus event owner and also forward the event to the IBus daemon with
+     *    the "ProcessKeyEvent" D-Bus method and wait for the return value of
+     *    the D-Bus method *asynchronously*.
+     *    If the return value indicates the key event is disposed by IBus,
+     *    the IBus client does not perform anything. Otherwise the IBus client
+     *    forwards the key event with the gdk_event_put() in GTK3,
+     *    gtk_im_context_filter_key() in GTK4, IMForwardEvent() in XIM API.
+     * 2: Hybrid asynchronous process key event:
+     *    Wait for the return of the IBus "ProcessKeyEvent" D-Bus method
+     *    *asynchronously* with a GSource loop and forward the return value
+     *    to the IBus event owner synchronously. So IBus clients perform
+     *    virtually synchronously to cover problems of IBus synchronous APIs.
+     *
+     * The purpose of the asynchronous process is that each IBus input
+     * method can process the key events without D-Bus timeout and also
+     * the IBus synchronous process has a problem that the IBus
+     * "ProcessKeyEvent" D-Bus method cannot send the commit-text and
+     * forwar-key-event D-Bus signals until the D-Bus method is finished.
+     *
+     * Relative issues: #1713, #2486
+     */
+    _use_sync_mode = _get_char_env ("IBUS_ENABLE_SYNC_MODE", 1);
 #else
     _use_key_snooper = !_get_boolean_env ("IBUS_DISABLE_SNOOPER",
                                           !(ENABLE_SNOOPER));
@@ -1004,8 +1107,6 @@ ibus_im_context_init (GObject *obj)
 #else
     ibusimcontext->caps = IBUS_CAP_PREEDIT_TEXT | IBUS_CAP_FOCUS;
 #endif
-    if (_use_sync_mode == 1)
-        ibusimcontext->caps |= IBUS_CAP_SYNC_PROCESS_KEY_V2;
 
     ibusimcontext->events_queue = g_queue_new ();
 
@@ -1110,6 +1211,7 @@ static void
 ibus_im_context_clear_preedit_text (IBusIMContext *ibusimcontext)
 {
     gchar *preedit_string = NULL;
+    IBusText *text;
     g_assert (ibusimcontext->ibuscontext);
     if (ibusimcontext->preedit_visible &&
         ibusimcontext->preedit_mode == IBUS_ENGINE_PREEDIT_COMMIT) {
@@ -1123,12 +1225,18 @@ ibus_im_context_clear_preedit_text (IBusIMContext *ibusimcontext)
      * would be located on the URL bar and click on anywhere of firefox
      * out of the URL bar.
      */
+    if (!(text = ibus_text_new_from_string (""))) {
+        g_warning ("Cannot allocate IBusText.");
+        g_free (preedit_string);
+        return;
+    }
     _ibus_context_update_preedit_text_cb (ibusimcontext->ibuscontext,
-                                          ibus_text_new_from_string (""),
+                                          text,
                                           ibusimcontext->preedit_cursor_pos,
                                           ibusimcontext->preedit_visible,
                                           IBUS_ENGINE_PREEDIT_CLEAR,
                                           ibusimcontext);
+    g_object_unref (text);
     if (preedit_string) {
         g_signal_emit (ibusimcontext, _signal_commit_id, 0, preedit_string);
         g_free (preedit_string);
@@ -1518,8 +1626,13 @@ static gboolean
 _set_cursor_location_internal (IBusIMContext *ibusimcontext)
 {
     GdkRectangle area;
+    GdkDisplay *display = NULL;
 #if GTK_CHECK_VERSION (3, 98, 4)
     GtkWidget *root;
+    GtkNative *native;
+    graphene_point_t p;
+    int tx = 0, ty = 0;
+    double nx = 0., ny = 0.;
 #endif
 
     if(ibusimcontext->client_window == NULL ||
@@ -1529,103 +1642,74 @@ _set_cursor_location_internal (IBusIMContext *ibusimcontext)
 
     area = ibusimcontext->cursor_area;
 
-#ifdef GDK_WINDOWING_WAYLAND
 #if GTK_CHECK_VERSION (3, 98, 4)
     root = GTK_WIDGET (gtk_widget_get_root (ibusimcontext->client_window));
-     /* FIXME: GTK_STYLE_CLASS_TITLEBAR is available in GTK3 but not GTK4.
-      * gtk_css_boxes_get_content_rect() is available in GTK4 but it's an
-      * internal API and calculate the window edge 32 in GTK3.
-      */
-    area.y += 32;
-    area.width = 50; /* FIXME: Why 50 meets the cursor position? */
-    area.height = gtk_widget_get_height (root);
-    area.height += 32;
-    if (GDK_IS_WAYLAND_DISPLAY (gdk_display_get_default ())) {
-        ibus_input_context_set_cursor_location_relative (
-            ibusimcontext->ibuscontext,
-            area.x,
-            area.y,
-            area.width,
-            area.height);
-        return FALSE;
+    /* Translates the given point in client_window coordinates to coordinates
+       relative to root coordinate system. */
+    if (!gtk_widget_compute_point (ibusimcontext->client_window,
+                                   root,
+                                   &GRAPHENE_POINT_INIT (area.x, area.y),
+                                   &p)) {
+        graphene_point_init (&p, area.x, area.y);
     }
-#else
-    if (GDK_IS_WAYLAND_DISPLAY (gdk_display_get_default ())) {
-        gdouble px, py;
-        GdkWindow *parent;
-        GdkWindow *window = ibusimcontext->client_window;
 
-        while ((parent = gdk_window_get_effective_parent (window)) != NULL) {
-            gdk_window_coords_to_parent (window, area.x, area.y, &px, &py);
-            area.x = px;
-            area.y = py;
-            window = parent;
-        }
+    native = gtk_widget_get_native (ibusimcontext->client_window);
+    /* Translates from the surface coordinates into the widget coordinates. */
+    gtk_native_get_surface_transform (native, &nx, &ny);
 
-        _set_rect_scale_factor_with_window (&area,
-                                            ibusimcontext->client_window);
-        ibus_input_context_set_cursor_location_relative (
-            ibusimcontext->ibuscontext,
-            area.x,
-            area.y,
-            area.width,
-            area.height);
-        return FALSE;
-    }
-#endif
-#endif
-
-#if GTK_CHECK_VERSION (3, 98, 4)
-#elif GTK_CHECK_VERSION (2, 91, 0)
-    if (area.x == -1 && area.y == -1 && area.width == 0 && area.height == 0) {
-        area.x = 0;
-        area.y += gdk_window_get_height (ibusimcontext->client_window);
-    }
-#else
-    if (area.x == -1 && area.y == -1 && area.width == 0 && area.height == 0) {
-        gint w, h;
-        gdk_drawable_get_size (ibusimcontext->client_window, &w, &h);
-        area.y += h;
-        area.x = 0;
-    }
-#endif
-
-#if GTK_CHECK_VERSION (3, 98, 4)
-#if defined(GDK_WINDOWING_X11)
-    GdkDisplay *display = gtk_widget_get_display (ibusimcontext->client_window);
+#ifdef HAVE_XIM
+    display = gtk_widget_get_display (ibusimcontext->client_window);
     if (GDK_IS_X11_DISPLAY (display)) {
-        Display *xdisplay = gdk_x11_display_get_xdisplay (display);
-        Window root_window = gdk_x11_display_get_xrootwindow (display);
-        GtkNative *native = gtk_widget_get_native (
-                ibusimcontext->client_window);
-        GdkSurface *surface = gtk_native_get_surface (native);
-        /* The window is the toplevel window but not the inner text widget.
-         * Unfortunatelly GTK4 cannot get the coordinate of the text widget.
-         */
-        Window window = gdk_x11_surface_get_xid (surface);
+        GdkSurface *surface = gtk_native_get_surface
+            (gtk_widget_get_native (ibusimcontext->client_window));
         Window child;
-        int x, y;
-        XTranslateCoordinates (xdisplay, window, root_window,
-                               0, 0, &x, &y, &child);
-        XWindowAttributes xwa;
-        XGetWindowAttributes (xdisplay, window, &xwa);
-        area.x = x - xwa.x + area.x;
-        area.y = y - xwa.y + area.y;
-        area.width = 50; /* FIXME: Why 50 meets the cursor position? */
-        area.height = xwa.height;
+        int scale_factor = gtk_widget_get_scale_factor
+            (ibusimcontext->client_window);
+
+        XTranslateCoordinates (GDK_DISPLAY_XDISPLAY (display),
+                               GDK_SURFACE_XID (surface),
+                               gdk_x11_display_get_xrootwindow (display),
+                               0, 0, &tx, &ty,
+                               &child);
+
+        tx = tx / scale_factor;
+        ty = ty / scale_factor;
     }
 #endif
+
+    area.x = p.x + nx + tx;
+    area.y = p.y + ny + ty;
 #else
     gdk_window_get_root_coords (ibusimcontext->client_window,
                                 area.x, area.y,
                                 &area.x, &area.y);
 #endif
+
     _set_rect_scale_factor_with_window (&area, ibusimcontext->client_window);
-    ibus_input_context_set_cursor_location (ibusimcontext->ibuscontext,
-                                            area.x,
-                                            area.y,
-                                            area.width,
-                                            area.height);
+
+#ifdef GDK_WINDOWING_WAYLAND
+#if !GTK_CHECK_VERSION (3, 98, 4)
+    display = gdk_window_get_display (ibusimcontext->client_window);
+#endif
+
+    if (GDK_IS_WAYLAND_DISPLAY (display)) {
+        ibus_input_context_set_cursor_location_relative (
+                ibusimcontext->ibuscontext,
+                area.x,
+                area.y,
+                area.width,
+                area.height);
+    } else {
+#endif
+        ibus_input_context_set_cursor_location (ibusimcontext->ibuscontext,
+                                                area.x,
+                                                area.y,
+                                                area.width,
+                                                area.height);
+#ifdef GDK_WINDOWING_WAYLAND
+    }
+#endif
+
     return FALSE;
 }
 
@@ -2265,6 +2349,8 @@ _create_input_context_done (IBusBus       *bus,
     else {
         gboolean requested_surrounding_text = FALSE;
         ibus_input_context_set_client_commit_preedit (context, TRUE);
+        if (_use_sync_mode == 1)
+            ibus_input_context_set_post_process_key_event (context, TRUE);
         ibusimcontext->ibuscontext = context;
 
         g_signal_connect (ibusimcontext->ibuscontext,
@@ -2346,7 +2432,7 @@ _create_input_context_done (IBusBus       *bus,
 static void
 _create_input_context (IBusIMContext *ibusimcontext)
 {
-    gchar *prgname = g_strdup (g_get_prgname());
+    gchar *prgname;
     gchar *client_name;
     IDEBUG ("%s", __FUNCTION__);
 
@@ -2354,6 +2440,7 @@ _create_input_context (IBusIMContext *ibusimcontext)
 
     g_return_if_fail (ibusimcontext->cancellable == NULL);
 
+    prgname = g_strdup (g_get_prgname());
     ibusimcontext->cancellable = g_cancellable_new ();
 
     if (!prgname)
@@ -2489,9 +2576,8 @@ _create_fake_input_context_done (IBusBus       *bus,
                       G_CALLBACK (_ibus_fake_context_destroy_cb),
                       NULL);
 
-    guint32 caps = IBUS_CAP_PREEDIT_TEXT | IBUS_CAP_FOCUS | IBUS_CAP_SURROUNDING_TEXT;
-    if (_use_sync_mode == 1)
-        caps |= IBUS_CAP_SYNC_PROCESS_KEY_V2;
+    guint32 caps = IBUS_CAP_PREEDIT_TEXT | IBUS_CAP_FOCUS
+                   | IBUS_CAP_SURROUNDING_TEXT;
     ibus_input_context_set_capabilities (_fake_context, caps);
 
     /* focus in/out the fake context */
