@@ -2,7 +2,7 @@
 /* vim:set et sts=4: */
 /* ibus - The Input Bus
  * Copyright (C) 2008-2015 Peng Huang <shawn.p.huang@gmail.com>
- * Copyright (C) 2015-2019 Takao Fujiwara <takao.fujiwara1@gmail.com>
+ * Copyright (C) 2015-2025 Takao Fujiwara <takao.fujiwara1@gmail.com>
  * Copyright (C) 2008-2016 Red Hat, Inc.
  *
  * This library is free software; you can redistribute it and/or
@@ -21,13 +21,14 @@
  * USA
  */
 
-#include "ibusbus.h"
 #include <errno.h>
+#include <fcntl.h>
 #include <sys/types.h>
 #include <sys/stat.h>
 #include <unistd.h>
 #include <glib/gstdio.h>
 #include <gio/gio.h>
+#include "ibusbus.h"
 #include "ibusmarshalers.h"
 #include "ibusinternal.h"
 #include "ibusshare.h"
@@ -43,6 +44,7 @@ enum {
     DISCONNECTED,
     GLOBAL_ENGINE_CHANGED,
     NAME_OWNER_CHANGED,
+    GLOBAL_SHORTCUT_KEY_RESPONDED,
     LAST_SIGNAL,
 };
 
@@ -60,7 +62,8 @@ struct _IBusBusPrivate {
     gboolean watch_dbus_signal;
     guint watch_dbus_signal_id;
     gboolean watch_ibus_signal;
-    guint watch_ibus_signal_id;
+    guint watch_global_engine_changed_id;
+    guint watch_global_shortcut_key_responded_id;
     IBusConfig *config;
     gchar *unique_name;
     gboolean connect_async;
@@ -234,6 +237,34 @@ ibus_bus_class_init (IBusBusClass *class)
             G_TYPE_NONE,
             3,
             G_TYPE_STRING, G_TYPE_STRING, G_TYPE_STRING);
+
+    /**
+     * IBusBus::global-shortcut-key-responded:
+     * @bus: The #IBusBus object which recevied the signal
+     * @type: The type of the global shortcut key.
+     * @keyval: Key symbol of the key press.
+     * @keycode: KeyCode of the key press.
+     * @state: Key modifier flags.
+     * @is_backward: %TRUE if the backward key is pressed.
+     *
+     * Emitted when global shortcut key is responded.
+     * Since 1.5.32
+     *
+     */
+    bus_signals[GLOBAL_SHORTCUT_KEY_RESPONDED] =
+        g_signal_new (I_("global-shortcut-key-responded"),
+            G_TYPE_FROM_CLASS (class),
+            G_SIGNAL_RUN_LAST,
+            0,
+            NULL, NULL,
+            _ibus_marshal_VOID__UCHAR_UINT_UINT_UINT_BOOLEAN,
+            G_TYPE_NONE,
+            5,
+            G_TYPE_UCHAR,
+            G_TYPE_UINT,
+            G_TYPE_UINT,
+            G_TYPE_UINT,
+            G_TYPE_BOOLEAN);
 }
 
 static void
@@ -272,12 +303,23 @@ _connection_ibus_signal_cb (GDBusConnection *connection,
     g_return_if_fail (user_data != NULL);
     g_return_if_fail (IBUS_IS_BUS (user_data));
 
-    if (g_strcmp0 (signal_name, "GlobalEngineChanged") == 0) {
+    if (!g_strcmp0 (signal_name, "GlobalEngineChanged")) {
         gchar *engine_name = NULL;
         g_variant_get (parameters, "(&s)", &engine_name);
         g_signal_emit (IBUS_BUS (user_data),
                        bus_signals[GLOBAL_ENGINE_CHANGED], 0,
                        engine_name);
+    } else if (!g_strcmp0 (signal_name, "GlobalShortcutKeyResponded")) {
+        guchar type = (guchar)IBUS_BUS_GLOBAL_BINDING_TYPE_ANY;
+        guint keyval = 0;
+        guint keycode = 0;
+        guint state = 0;
+        gboolean is_backward = FALSE;
+        g_variant_get (parameters, "(yuuub)",
+                       &type, &keyval, &keycode, &state, &is_backward);
+        g_signal_emit (IBUS_BUS (user_data),
+                       bus_signals[GLOBAL_SHORTCUT_KEY_RESPONDED], 0,
+                       type, keyval, keycode, state, is_backward);
     }
     /* FIXME handle org.freedesktop.IBus.RegistryChanged signal if needed */
 }
@@ -307,7 +349,7 @@ ibus_bus_close_connection (IBusBus *bus)
     bus->priv->unique_name = NULL;
 
     bus->priv->watch_dbus_signal_id = 0;
-    bus->priv->watch_ibus_signal_id = 0;
+    bus->priv->watch_global_engine_changed_id = 0;
 
     g_free (bus->priv->bus_address);
     bus->priv->bus_address = NULL;
@@ -537,7 +579,8 @@ static void
 ibus_bus_init (IBusBus *bus)
 {
     struct stat buf;
-    gchar *path;
+    char *path;
+    int fd;
 
     bus->priv = IBUS_BUS_GET_PRIVATE (bus);
 
@@ -546,7 +589,8 @@ ibus_bus_init (IBusBus *bus)
     bus->priv->watch_dbus_signal = FALSE;
     bus->priv->watch_dbus_signal_id = 0;
     bus->priv->watch_ibus_signal = FALSE;
-    bus->priv->watch_ibus_signal_id = 0;
+    bus->priv->watch_global_engine_changed_id = 0;
+    bus->priv->watch_global_shortcut_key_responded_id = 0;
     bus->priv->unique_name = NULL;
     bus->priv->connect_async = FALSE;
     bus->priv->client_only = FALSE;
@@ -555,21 +599,38 @@ ibus_bus_init (IBusBus *bus)
 
     path = g_path_get_dirname (ibus_get_socket_path ());
 
-    g_mkdir_with_parents (path, 0700);
+    errno = 0;
+    if (g_mkdir_with_parents (path, 0700)) {
+        g_warning ("Failed to mkdir %s: %s", path, g_strerror (errno));
+        g_free (path);
+        return;
+    }
 
-    if (stat (path, &buf) == 0) {
+    errno = 0;
+    if ((fd = open (path, O_RDONLY | O_DIRECTORY, S_IRWXU)) == -1) {
+        g_warning ("open %s failed: %s", path, g_strerror (errno));
+        g_free (path);
+        return;
+    }
+    /* TOCTOU: Use fstat() and fchmod() but not stat() and chmod().
+     * because it can cause a time-of-check, time-of-use race condition.
+     */
+    if (fstat (fd, &buf) == 0) {
         if (buf.st_uid != getuid ()) {
             g_warning ("The owner of %s is not %s!",
                        path, ibus_get_user_name ());
+            close (fd);
+            g_free (path);
             return;
         }
         if (buf.st_mode != (S_IFDIR | S_IRWXU)) {
             errno = 0;
-            if (g_chmod (path, 0700))
-                g_warning ("chmod failed: %s", errno ? g_strerror (errno) : "");
+            if (fchmod (fd, S_IRWXU))
+                g_warning ("chmod failed: %s", g_strerror (errno));
         }
     }
 
+    close (fd);
     g_free (path);
 }
 
@@ -1136,10 +1197,10 @@ static void
 ibus_bus_watch_ibus_signal (IBusBus *bus)
 {
     g_return_if_fail (bus->priv->connection != NULL);
-    g_return_if_fail (bus->priv->watch_ibus_signal_id == 0);
+    g_return_if_fail (bus->priv->watch_global_engine_changed_id == 0);
 
     /* Subscribe to ibus signals such as GlboalEngineChanged. */
-    bus->priv->watch_ibus_signal_id
+    bus->priv->watch_global_engine_changed_id
         = g_dbus_connection_signal_subscribe (bus->priv->connection,
                                               "org.freedesktop.IBus",
                                               IBUS_INTERFACE_IBUS,
@@ -1150,16 +1211,32 @@ ibus_bus_watch_ibus_signal (IBusBus *bus)
                                               _connection_ibus_signal_cb,
                                               bus,
                                               NULL /* user_data_free_func */);
+    bus->priv->watch_global_shortcut_key_responded_id
+        = g_dbus_connection_signal_subscribe (bus->priv->connection,
+                                              "org.freedesktop.IBus",
+                                              IBUS_INTERFACE_IBUS,
+                                              "GlobalShortcutKeyResponded",
+                                              IBUS_PATH_IBUS,
+                                              NULL /* arg0 */,
+                                              (GDBusSignalFlags) 0,
+                                              _connection_ibus_signal_cb,
+                                              bus,
+                                              NULL /* user_data_free_func */);
     /* FIXME handle org.freedesktop.IBus.RegistryChanged signal if needed */
 }
 
 static void
 ibus_bus_unwatch_ibus_signal (IBusBus *bus)
 {
-    g_return_if_fail (bus->priv->watch_ibus_signal_id != 0);
-    g_dbus_connection_signal_unsubscribe (bus->priv->connection,
-                                          bus->priv->watch_ibus_signal_id);
-    bus->priv->watch_ibus_signal_id = 0;
+    g_return_if_fail (bus->priv->watch_global_engine_changed_id != 0);
+    g_dbus_connection_signal_unsubscribe (
+            bus->priv->connection,
+            bus->priv->watch_global_engine_changed_id);
+    bus->priv->watch_global_engine_changed_id = 0;
+    g_dbus_connection_signal_unsubscribe (
+            bus->priv->connection,
+            bus->priv->watch_global_shortcut_key_responded_id);
+    bus->priv->watch_global_shortcut_key_responded_id = 0;
 }
 
 void
@@ -2394,6 +2471,109 @@ ibus_bus_preload_engines_async_finish (IBusBus       *bus,
     return _async_finish_void (task, error);
 }
 
+static GVariant *
+ibus_process_key_event_data_list_serialize (
+        IBusBusGlobalBindingType       gtype,
+        const IBusProcessKeyEventData *keys)
+{
+    GVariantBuilder builder, array;
+    gsize i;
+
+    g_variant_builder_init (&builder, G_VARIANT_TYPE_TUPLE);
+    g_variant_builder_add (&builder, "y", gtype);
+    g_variant_builder_init (&array, G_VARIANT_TYPE ("a(uuu)"));
+    for (i = 0; keys[i].keyval; ++i) {
+        g_variant_builder_add (&array, "(uuu)",
+                               keys[i].keyval,
+                               keys[i].keycode,
+                               keys[i].state);
+    }
+    g_variant_builder_add (&builder, "a(uuu)", &array);
+    return g_variant_builder_end (&builder);
+}
+
+gboolean
+ibus_bus_set_global_shortcut_keys (IBusBus                       *bus,
+                                   IBusBusGlobalBindingType       gtype,
+                                   const IBusProcessKeyEventData *keys)
+{
+    GVariant *result;
+    GVariant *variant = NULL;
+
+    g_return_val_if_fail (IBUS_IS_BUS (bus), FALSE);
+    g_return_val_if_fail (keys != NULL && keys[0].keyval != 0, FALSE);
+
+    variant = ibus_process_key_event_data_list_serialize (gtype, keys);
+    result = ibus_bus_call_sync (bus,
+                                 IBUS_SERVICE_IBUS,
+                                 IBUS_PATH_IBUS,
+                                 "org.freedesktop.DBus.Properties",
+                                 "Set",
+                                 g_variant_new ("(ssv)",
+                                                IBUS_INTERFACE_IBUS,
+                                                "GlobalShortcutKeys",
+                                                variant),
+                                 NULL);
+
+    if (result) {
+        g_variant_unref (result);
+        return TRUE;
+    }
+
+    return FALSE;
+}
+
+void
+ibus_bus_set_global_shortcut_keys_async (IBusBus                       *bus,
+                                         IBusBusGlobalBindingType       gtype,
+                                         const IBusProcessKeyEventData *keys,
+                                         gint
+                                                                   timeout_msec,
+                                         GCancellable
+                                                                  *cancellable,
+                                         GAsyncReadyCallback
+                                                                   callback,
+                                         gpointer                  user_data)
+{
+    GVariant *variant = NULL;
+
+    g_return_if_fail (IBUS_IS_BUS (bus));
+    g_return_if_fail (keys != NULL && keys[0].keyval != 0);
+
+    variant = ibus_process_key_event_data_list_serialize (gtype, keys);
+    ibus_bus_call_async (bus,
+                         IBUS_SERVICE_IBUS,
+                         IBUS_PATH_IBUS,
+                         "org.freedesktop.DBus.Properties",
+                         "Set",
+                         g_variant_new ("(ssv)",
+                                        IBUS_INTERFACE_IBUS,
+                                        "GlobalShortcutKeys",
+                                        variant),
+                         NULL, /* no return value */
+                         ibus_bus_set_global_shortcut_keys_async,
+                         timeout_msec,
+                         cancellable,
+                         callback,
+                         user_data);
+}
+
+gboolean
+ibus_bus_set_global_shortcut_keys_async_finish (IBusBus       *bus,
+                                                GAsyncResult  *res,
+                                                GError       **error)
+{
+    GTask *task;
+
+    g_assert (IBUS_IS_BUS (bus));
+    g_assert (g_task_is_valid (res, bus));
+
+    task = G_TASK (res);
+    g_assert (g_task_get_source_tag (task) ==
+              ibus_bus_set_global_shortcut_keys_async);
+    return _async_finish_void (task, error);
+}
+
 GVariant *
 ibus_bus_get_ibus_property (IBusBus     *bus,
                             const gchar *property_name)
