@@ -802,20 +802,29 @@ start_set_preload_engines_async (void)
             NULL); /* user_data */
 }
 
+typedef struct _ExitAsyncData {
+    gboolean has_socket_path;
+    gboolean exited;
+    guint    timeout_id;
+} ExitAsyncData;
+
 static void
 _socket_changed_cb (GFileMonitor       *monitor,
                     GFile              *file,
                     GFile              *other_file,
                     GFileMonitorEvent   event_type,
-                    IBusBus            *bus)
+                    ExitAsyncData      *data)
 {
     switch (event_type) {
     case G_FILE_MONITOR_EVENT_CHANGED:
         g_debug ("IBus socket file is changed");
         call_next_async_function ();
+        data->exited = TRUE;
         g_signal_handlers_disconnect_by_func (monitor,
                                               G_CALLBACK (_socket_changed_cb),
-                                              NULL);
+                                              data);
+        if (data->timeout_id)
+            g_source_remove (data->timeout_id);
         g_object_unref (monitor);
         break;
     case G_FILE_MONITOR_EVENT_CREATED:
@@ -829,6 +838,33 @@ _socket_changed_cb (GFileMonitor       *monitor,
     }
 }
 
+static gboolean
+_exit_timeout (gpointer user_data)
+{
+    g_error ("start_exit_async() is timeout. You might run ibus-daemon " \
+             "with systemd under GNOME and the exit API does not work. " \
+             "You need to export IBUS_DAEMON_WITH_SYSTEMD=1 .\n");
+    return G_SOURCE_REMOVE;
+}
+
+static void
+finish_ibus_restart_async (GPid      pid,
+                           gint      status,
+                           gpointer *user_data)
+{
+    ExitAsyncData *data = (ExitAsyncData *)user_data;
+    g_spawn_close_pid (pid);
+    if (data->has_socket_path == FALSE) {
+        g_debug ("ibus_bus_exit_finish: OK socket file: none");
+        g_usleep (G_USEC_PER_SEC);
+        call_next_async_function ();
+    } else {
+        g_debug ("ibus_bus_exit_finish: OK socket file: monitored");
+        if (!data->exited)
+            data->timeout_id = g_timeout_add_seconds (10, _exit_timeout, NULL);
+    }
+}
+
 static void
 finish_exit_async (GObject *source_object,
                    GAsyncResult *res,
@@ -838,25 +874,32 @@ finish_exit_async (GObject *source_object,
     gboolean result = ibus_bus_exit_async_finish (bus,
                                                   res,
                                                   &error);
-    gboolean has_socket_path = GPOINTER_TO_INT (user_data);
+    ExitAsyncData *data = (ExitAsyncData *)user_data;
     if (error) {
         g_warning ("Failed to ibus_bus_exit(): %s", error->message);
         g_error_free (error);
     }
     g_assert (result);
-    if (has_socket_path == FALSE) {
+    g_assert (data);
+    if (data->has_socket_path == FALSE) {
         g_debug ("ibus_bus_exit_finish: OK socket file: none");
         g_usleep (G_USEC_PER_SEC);
         call_next_async_function ();
     } else {
         g_debug ("ibus_bus_exit_finish: OK socket file: monitored");
+        if (!data->exited)
+            data->timeout_id = g_timeout_add_seconds (10, _exit_timeout, NULL);
     }
 }
 
 static void
 start_exit_async (void)
 {
-    gboolean has_socket_path = FALSE;
+    static ExitAsyncData data = {
+        .has_socket_path = FALSE,
+        .exited          = FALSE,
+        .timeout_id      = 0
+    };
     /* When `./runtest ibus-bus` runs, ibus-daemon sometimes failed to
      * restart because closing a file descriptor was failed in
      * bus/server.c:_restart_server() with a following error:
@@ -879,7 +922,7 @@ start_exit_async (void)
         g_assert (address_path);
         file = g_file_new_for_path (address_path);
         g_assert (file);
-        has_socket_path = TRUE;
+        data.has_socket_path = TRUE;
         monitor = g_file_monitor (file, G_FILE_MONITOR_NONE, NULL, &error);
         if (error) {
             g_warning ("Failed to monitor socket file: %s", error->message);
@@ -887,15 +930,38 @@ start_exit_async (void)
         }
         g_assert (monitor);
         g_signal_connect (monitor, "changed",
-                          G_CALLBACK (_socket_changed_cb), NULL);
+                          G_CALLBACK (_socket_changed_cb),
+                          &data);
         g_object_unref (file);
     }
-    ibus_bus_exit_async (bus,
-                         TRUE, /* restart */
-                         -1, /* timeout */
-                         NULL, /* cancellable */
-                         finish_exit_async,
-                         GINT_TO_POINTER (has_socket_path)); /* user_data */
+    /* When ibus-daemon runs with systemd, restarting the daemon with
+     * ibus_bus_exit_async() does not work so runs `ibus restart` command
+     * with IBUS_DAEMON_WITH_SYSTEMD variable instead.
+     */
+    if (g_getenv ("IBUS_DAEMON_WITH_SYSTEMD")) {
+        gchar *argv[] = { "ibus", "restart", NULL };
+        GSpawnFlags flags = G_SPAWN_DO_NOT_REAP_CHILD \
+                            | G_SPAWN_SEARCH_PATH \
+                            | G_SPAWN_STDOUT_TO_DEV_NULL \
+                            | G_SPAWN_STDERR_TO_DEV_NULL;
+        GPid pid = 0;
+        GError *error = NULL;
+        g_spawn_async (NULL, argv, NULL, flags, NULL, NULL, &pid, &error);
+        if (error) {
+            g_warning ("Failed to call ibus restart: %s", error->message);
+            g_error_free (error);
+        }
+        g_child_watch_add (pid,
+                           (GChildWatchFunc)finish_ibus_restart_async,
+                           &data);
+    } else {
+        ibus_bus_exit_async (bus,
+                             TRUE, /* restart */
+                             -1, /* timeout */
+                             NULL, /* cancellable */
+                             finish_exit_async,
+                             &data); /* user_data */
+    }
 }
 
 static gboolean
