@@ -4,10 +4,17 @@
 static void
 test_machine_id (void)
 {
+    DBusError error = DBUS_ERROR_INIT;
     const gchar *s1 = ibus_get_local_machine_id ();
-    gchar *s2 = dbus_get_local_machine_id ();
+    gchar *s2 = dbus_try_get_local_machine_id (&error);
 
-    g_assert_cmpstr (s1, ==, s2);
+    g_assert (s1);
+    if (!s2) {
+        g_error ("Failed to get the local machine id: %s: %s",
+                 error.name, error.message);
+    } else {
+        g_assert_cmpstr (s1, ==, s2);
+    }
     dbus_free (s2);
 }
 
@@ -15,8 +22,8 @@ gint
 main (gint    argc,
       gchar **argv)
 {
-    g_mem_set_vtable (glib_mem_profiler_table);
-	g_type_init ();
+    ibus_init ();
+
     g_test_init (&argc, &argv, NULL);
     g_test_add_func ("/ibus/marchine-id", test_machine_id);
 
